//*****************************************************************************
// Copyright (c) 2006, Freescale Semiconductor
// Freescale Confidential Proprietary
// For use on Freescale products only.
//
// Project name: 	emg_HTTP web server for Coldfire
// Author:			Eric Gregori   (847) 651 - 1971
//		   			eric.gregori@freescale.com
//
//
//*****************************************************************************
#include <stdio.h>
#include <stdlib.h>
#include <string.h>



//**************************************************************************
// Defines
//**************************************************************************
#define MAX_FILENAME_LENGTH			255
#define MAX_NUMBER_OF_FILES			255
#define FFS_REVISION_NUMBER			'EMG1'
#define ARGC_INPUT_FILENAME			1
#define ARGC_OUTPUT_FILENAME		2
#define ARGC_PROTOCOL				3
#define TRUE						1
#define FALSE						0

//**************************************************************************
// Default HTTP file header
//**************************************************************************
#define HTTP_OK_RESPONSE			"HTTP/1.%c 200 OK\r\n"
#define HTTP_SERVER_NAME			"Server: EMG/1.1.0\r\n"
#define HTTP_CACHE_CONTROL			"Cache-control: no-cache\r\n"
#define HTTP_TITLE					"HTTP Server By Eric Gregori Version 1.1.0\r\n"
#define HTTP_CONTENT_TYPE			"Content-type: "
#define HTTP_CONTENT_LENGTH			"Content-length: "

#define HTTP_HTM					"text/html"
#define HTTP_TXT					"text/plain"
#define HTTP_GIF					"image/gif"
#define HTTP_XBM					"image/x-xbitmap"
#define HTTP_WAV					"audio/wav"
#define HTTP_JPEG					"image/jpeg"
#define HTTP_JAR					"application/java-archive"
#define HTTP_JS						"application/x-javascript"
#define HTTP_SWF					"application/x-shockwave-flash"
#define HTTP_PNG					"image/png"
#define HTTP_XML					"text/xml"
#define HTTP_BMP					"image/bmp"


//**************************************************************************
// FAT Data Structures
//**************************************************************************
typedef struct	{
						unsigned long	fatID;
						unsigned long	total_size_in_bytes;
						unsigned long	number_of_files;
						unsigned long	fat_pointer;
				}		FAT_HEADER;

typedef struct	{
						unsigned long	file_pointer;
						unsigned long	file_size_in_bytes;
						unsigned long	file_type;
				}		FAT_FILE_DESCRIPTOR; 

FAT_HEADER				fat_header;
FAT_FILE_DESCRIPTOR		fat_file_descriptors[MAX_NUMBER_OF_FILES];


char					filenames[MAX_NUMBER_OF_FILES*MAX_FILENAME_LENGTH];


//**************************************************************************
// Forward Declarations
//**************************************************************************
void			print_data(FILE *output, int bytenum, int data );
unsigned long	copy_file_to_output( FILE *input, FILE *output );
unsigned long	write_header( FILE *output, unsigned char *filename, unsigned long *filetype );
unsigned long	big_endian( unsigned long little_endian );


//**************************************************************************
// Global Vars
//**************************************************************************
unsigned char	filelist[MAX_NUMBER_OF_FILES][MAX_FILENAME_LENGTH];
unsigned char	http_protocol;


//**************************************************************************
// Main
//
//		1st Pass
//					Read in each file, get length, build HTTP header, add http header
//					write file to ????.htp
//
//		2nd Pass
//					Write header, copy .htp files to output
//
// Author: Eric Gregori
//**************************************************************************
void main( int argc, char *argv[])
{
	FILE			*input_file, *output_file, *data_file, *htp_file; 
	unsigned short	i;
	unsigned short	number_of_files;
	unsigned char	tempstorage[MAX_FILENAME_LENGTH];	/* should probably malloc ... */
	unsigned char	tempstorage2[MAX_FILENAME_LENGTH];
	unsigned long	htp_length, total_length, file_size, filenameheaderlength;


	http_protocol = '1';
	printf("\nDynamic Flash File System Generator, Revision %s %s",__DATE__, __TIME__);
	printf("\n\tWritten by Eric Gregori\n");

	if( argc < 3 )
	{
		//**************************************************************************
		// Usage Header
		//**************************************************************************
		printf("\nCorrect format:  <input filename> <output filename> <protocol>\n"); 
		printf("\n");
		printf("Where input filename = file containing list of filenames\n");
		printf("\n");
		printf("Where output filename = generated filename\n");
		printf("\n");
		printf("Where protocol = 1 or 0\n");
		return;
	}

	//**************************************************************************
	// Open input file, containing list of files to include
	//**************************************************************************
	if((input_file = fopen(argv[ARGC_INPUT_FILENAME], "r+b")) == 0) 
	{ 
		printf("Cannot open input file: %s\n", argv[ARGC_INPUT_FILENAME]); 
		return;
	}

	//**************************************************************************
	// Open Output File
	//**************************************************************************
	if((output_file = fopen(argv[ARGC_OUTPUT_FILENAME], "wb")) == 0) 
	{ 
		printf("Cannot open output file: %s\n", argv[ARGC_OUTPUT_FILENAME]); 
		return;
	}

	//**************************************************************************
	// Protocol
	//**************************************************************************
	if((argc == 4) && (argv[ARGC_PROTOCOL][0] == '0') ) 
	{ 
		http_protocol = '0';
		printf("\n\nOverriding Protocol to HTTP1.0" );
	}

	//**************************************************************************
	// 1st Pass,  loop through files and create .htp files with HTTP headers
	//**************************************************************************
	number_of_files = 0;
	total_length	= 0;
	filenames[0]	= 0;
	do
	{
		//**************************************************************************
		// Read in one CR terminated line from input file list
		//**************************************************************************
		if( fgets( tempstorage, MAX_FILENAME_LENGTH, input_file ) == NULL )
			break;

		//**************************************************************************
		// If the first char is a * or space, ignore the whole line ( used for comments )
		//**************************************************************************
		if( (tempstorage[0] != ' ') && (tempstorage[0] != '*') )
		{
			if( (strlen( tempstorage ) < MAX_FILENAME_LENGTH) &&
				(strlen( tempstorage ) > 3) )
			{
				//**************************************************************************
				// tempstorage is a path/filename - null terminate it
				//**************************************************************************
				for( i=0; tempstorage[i]; i++ )
				{
					if( (tempstorage[i] == 0x0D) || (tempstorage[i] == 0x0A) )
						tempstorage[i] = 0;
				}

				//**************************************************************************
				// Build filename list
				//**************************************************************************
				for( i=0; (filenames[i]||filenames[i+1]); i++ );
				i++;
				(void)sprintf( &filenames[i], "%s", tempstorage );

				//**************************************************************************
				// tempstorage2 is used to create the .htp path/filename
				//**************************************************************************
				strcpy( tempstorage2, tempstorage );
				for( i=0; tempstorage2[i]; i++ )
				{
					if( tempstorage2[i] == '.' )
						break;
				}

				tempstorage2[i]		= '_';

				for( ;tempstorage2[i]; i++ );

				tempstorage2[i]   = '.';
				tempstorage2[i+1] = 'h';
				tempstorage2[i+2] = 't';
				tempstorage2[i+3] = 'p';
				tempstorage2[i+4] = 0;

				//**************************************************************************
				// Open the .htp output file
				//**************************************************************************
				if((htp_file = fopen( tempstorage2, "wb")) == 0) 
				{ 
					printf("Cannot open output file: %s\n", tempstorage2); 
					return;
				}

				//**************************************************************************
				// Populate filelist so we only need to do this once.
				//**************************************************************************
				strcpy( filelist[number_of_files], tempstorage2 );

				//**************************************************************************
				// Write HTTP header to output file
				//**************************************************************************
				htp_length = write_header( htp_file, tempstorage, &fat_file_descriptors[number_of_files].file_type  );
				printf( "\nheader size = %d", htp_length );

				if((data_file = fopen(tempstorage, "rb")) != NULL) 
				{ 
					printf( "\nreading file: %s ", tempstorage );
					file_size = copy_file_to_output( data_file, htp_file );
					printf( "file size = %d", file_size );
					htp_length += file_size;
					total_length += htp_length;
					fclose( data_file );
					fat_file_descriptors[number_of_files].file_size_in_bytes = htp_length;
					number_of_files++;
				}
				else
				{
					printf("\nCannot open file: %s\n", tempstorage); 
				}

				fclose( htp_file );
			}
		}
	}while( !feof(input_file) );

	if( !number_of_files )
	{
		printf( "\n\nNo filenames found in input file" );
		return;
	}

	// we want to write a long aligned amount of bytes from filename
	for( i=0; ; i++ )
	{
		if( (filenames[i]   == 0) && 
			(filenames[i+1] == 0) &&
			(i%4            == 0) )
			break;
	}

	filenameheaderlength = i;

	printf( "\nfilename header length = 0x%x", filenameheaderlength );

	//**************************************************************************
	// Build header
	//**************************************************************************
	fat_header.fatID			= big_endian((unsigned long)FFS_REVISION_NUMBER);

	// Add header size to total length
	total_length += ((number_of_files*sizeof( FAT_FILE_DESCRIPTOR )) + sizeof( FAT_HEADER ) );
	total_length += filenameheaderlength;
	fat_header.total_size_in_bytes  = big_endian( total_length );	
	fat_header.number_of_files		= big_endian( number_of_files );
	fat_header.fat_pointer			= big_endian( sizeof( FAT_HEADER ) + filenameheaderlength );

	//**************************************************************************
	// Build File Descriptors
	//**************************************************************************
	fat_file_descriptors[0].file_pointer = 
		sizeof( FAT_HEADER ) + (sizeof( FAT_FILE_DESCRIPTOR ) * number_of_files)
		+ filenameheaderlength;
	for( i=1; i<number_of_files; i++ )
	{
		fat_file_descriptors[i].file_pointer = 
			fat_file_descriptors[i-1].file_pointer + 
			fat_file_descriptors[i-1].file_size_in_bytes;
	}

	//**************************************************************************
	// Convert from little to big endian
	//**************************************************************************
	for( i=0; i<number_of_files; i++ )
	{
		fat_file_descriptors[i].file_pointer		= big_endian( fat_file_descriptors[i].file_pointer );
		fat_file_descriptors[i].file_size_in_bytes	= big_endian( fat_file_descriptors[i].file_size_in_bytes );
		fat_file_descriptors[i].file_type			= big_endian( fat_file_descriptors[i].file_type );
	}

	//**************************************************************************
	// Write headers
	//**************************************************************************
	(void)fwrite( &fat_header, 1, sizeof( FAT_HEADER ), output_file );

	(void)fwrite( filenames, 1, filenameheaderlength, output_file );

	for( i=0; i<number_of_files; i++ )
		(void)fwrite( &fat_file_descriptors[i], 1, sizeof( FAT_FILE_DESCRIPTOR ), output_file );

	//**************************************************************************
	// Copy htp files to output
	//**************************************************************************
	for( i=0; i<number_of_files; i++ )
	{
		if((htp_file = fopen(filelist[i], "rb")) != NULL) 
		{ 
			printf( "\ncopying file: %s to output", filelist[i] );
			(void)copy_file_to_output( htp_file, output_file );
			fclose( htp_file );
		}
	}

	_fcloseall( );
}


//**************************************************************************
// copy_file_to_output( input FILE, output FILE )
//
//	copy binary input file to output file.
//
// Author: Eric Gregori
//**************************************************************************
unsigned long copy_file_to_output( FILE *input, FILE *output )
{
	unsigned long	length, total_count;
	unsigned char	temp[255];


	for( total_count=0; !feof(input); )
	{
		if ((length = fread(temp, 1, 255, input)) == 0)
			break;			

		if ( ferror(input) )
			break;

		(void)fwrite( temp, 1, length, output );
		total_count += length;
	}

	return( total_count );
}


//**************************************************************************
// write_header( output FILE, filename, *filetype )
//
//	Generate a HTTP header, and write it to the output FILE.
//	filename is used to open the file, and check the size.
//	The filetype pointer is used to return the filetype.
//
// Author: Eric Gregori
//**************************************************************************
unsigned long write_header( FILE *output, unsigned char *filename, unsigned long *filetype )
{
	FILE			*P_file;
	unsigned int	length, file_size;
	unsigned char	output_buffer[1024];
	unsigned long	buffer_index;


	// Header Fields
	//		HTTP_OK_RESPONSE
	//		HTTP_SERVER_NAME
	//		HTTP_CACHE_CONTROL
	//		HTTP_TITLE
	//		HTTP_CONTENT_TYPE
	buffer_index	= 0;
	buffer_index	+= sprintf( &output_buffer[buffer_index], HTTP_OK_RESPONSE, http_protocol );
	buffer_index	+= sprintf( &output_buffer[buffer_index], HTTP_SERVER_NAME );
	buffer_index	+= sprintf( &output_buffer[buffer_index], HTTP_CACHE_CONTROL );
	buffer_index	+= sprintf( &output_buffer[buffer_index], HTTP_TITLE );
	buffer_index	+= sprintf( &output_buffer[buffer_index], HTTP_CONTENT_TYPE );

	for( length=0; length<MAX_FILENAME_LENGTH; length++ )
	{
		if( filename[length] == '.' )
			break;
	}

	// "txt" = HTTP_TXT
	// "gif" = HTTP_GIF
	// "wav" = HTTP_WAV
	// "jpg" = HTTP_JPEG
	// "jar" = HTTP_JAR
	// "js"	 = HTTP_JS
	// "swf" = HTTP_SWF
	// "png" = HTTP_PNG
	// default = HTTP_HTM					
	if( strncmp( &filename[length+1], "txt", 3 ) == 0 )
	{
		buffer_index	+= sprintf( &output_buffer[buffer_index], HTTP_TXT );
		*filetype		= (unsigned long)'text';
	}
	else if( strncmp( &filename[length+1], "gif", 3 ) == 0 )
	{
		buffer_index	+= sprintf( &output_buffer[buffer_index], HTTP_GIF );
		*filetype		= (unsigned long)'giff';
	}
	else if( strncmp( &filename[length+1], "wav", 3 ) == 0 )
	{
		buffer_index	+= sprintf( &output_buffer[buffer_index], HTTP_WAV );
		*filetype		= (unsigned long)'wave';
	}
	else if( strncmp( &filename[length+1], "jpg", 3 ) == 0 )
	{
		buffer_index	+= sprintf( &output_buffer[buffer_index], HTTP_JPEG );
		*filetype		= (unsigned long)'jpeg';
	}
	else if( strncmp( &filename[length+1], "jar", 3 ) == 0 )
	{
		buffer_index	+= sprintf( &output_buffer[buffer_index], HTTP_JAR );
		*filetype		= (unsigned long)'jarr';
	}
	else if( strncmp( &filename[length+1], "js", 2 ) == 0 )
	{
		buffer_index	+= sprintf( &output_buffer[buffer_index], HTTP_JS );
		*filetype		= (unsigned long)'jsjs';
	}
	else if( strncmp( &filename[length+1], "swf", 3 ) == 0 )
	{
		buffer_index	+= sprintf( &output_buffer[buffer_index], HTTP_SWF );
		*filetype		= (unsigned long)'swff';
	}
	else if( strncmp( &filename[length+1], "png", 3 ) == 0 )
	{
		buffer_index	+= sprintf( &output_buffer[buffer_index], HTTP_PNG );
		*filetype		= (unsigned long)'pngg';
	}
	else if( strncmp( &filename[length+1], "xml", 3 ) == 0 )
	{
		buffer_index	+= sprintf( &output_buffer[buffer_index], HTTP_XML );
		*filetype		= (unsigned long)'xmll';
	}
	else if( strncmp( &filename[length+1], "bmp", 3 ) == 0 )
	{
		buffer_index	+= sprintf( &output_buffer[buffer_index], HTTP_BMP );
		*filetype		= (unsigned long)'bmpp';
	}
	else
	{
		buffer_index	+= sprintf( &output_buffer[buffer_index], HTTP_HTM );
		*filetype		= 'html';
	}

	buffer_index	+= sprintf( &output_buffer[buffer_index], "\r\n" );
	buffer_index	+= sprintf( &output_buffer[buffer_index], HTTP_CONTENT_LENGTH );
			
	P_file = fopen( filename, "rb" );
	if( P_file )
	{
  		fseek (P_file,0, SEEK_END);
  		file_size = ftell (P_file);

		/* complete header */
		buffer_index += sprintf( &output_buffer[buffer_index], "%d\r\n\r\n", file_size );
		rewind( P_file );
		fclose( P_file );
	}

	(void)fwrite( output_buffer, 1, buffer_index, output ); 

	return(buffer_index);
}


//**************************************************************************
// unsigned long big_endian( unsigned long little_endian )
//
//	Converts a 32 bit word from little to big endian.
//
// Author: Eric Gregori
//**************************************************************************
unsigned long	big_endian( unsigned long little_endian )
{
	unsigned long	temp;


	temp =  (unsigned long)((little_endian & 0xFF000000) >> 24);
	temp |= (unsigned long)((little_endian >> 8) & 0x0000FF00);
	temp |= (unsigned long)((little_endian << 24) & 0xFF000000);
	temp |= (unsigned long)((little_endian << 8) & 0x00FF0000);

	return( temp );
}
	
