//*****************************************************************************
// Copyright (c) 2006, Freescale Semiconductor
// Freescale Confidential Proprietary
// For use on Freescale products only.
//
// Project name: 	emg_HTTP web server for Coldfire
// Author:			Eric Gregori   (847) 651 - 1971
//		   			eric.gregori@freescale.com
//
//
//*****************************************************************************
#include <errno.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <errno.h>

#include <winsock2.h>


#define EMSGSIZE		WSAEMSGSIZE
#define INIT()			init( argv );
#define EXIT(s)			do { WSACleanup(); exit(s); } while(0)
#define CLOSE(s)		if( closesocket(s) ) error( 1, errno, "close failed" )
#define errno			( GetLastError() )
#define set_errno(e)	SetLastError( e )
#define isvalidsock(s)	((s) != SOCKET_ERROR )
#define bzero(b,n)		memset( b, 0, n )
#define sleep(t)		Sleep(t * 1000 )

#define PACKET_SIZE					0x100

#define ARGC_IP_ADDRESS				1
#define ARGC_INPUT_FILENAME			2
#define ARGC_KEY					3
 

char			*program_name;
unsigned int	total_bytes;



int inet_aton( char *cp, struct in_addr *pin )
{
	int			rc;

	rc = inet_addr( cp );
	if( rc == -1 && strcmp( cp, "255.255.255.255" ))
		return(0);
	pin->s_addr = rc;
	return(1);
}


void error( int status, int err, char *fmt, ... )
{
	va_list			ap;

	va_start( ap, fmt );
	fprintf( stderr, "%s: ", program_name );
	vfprintf( stderr, fmt, ap );
	va_end( ap );
	if( err )
		fprintf( stderr, ": %s (%d)\n", strerror( err ), err );
	if( status )
	{
		EXIT( status );
	}
}


static void set_address( char *hname, char *sname, struct sockaddr_in *sap, char *protocol )
{
	struct servent	*sp;
	struct hostent	*hp;
	char			*endptr;
	short			port;


	bzero( sap, sizeof( *sap ) );
	sap->sin_family = AF_INET;
	if( hname != NULL )
	{
		if( !inet_aton( hname, &sap->sin_addr ) )
		{
			hp = gethostbyname( hname );
			if( hp == NULL )
				error( 1, 0, "unknown host: %s\n", hname );
			sap->sin_addr = *( struct in_addr *)hp->h_addr_list;
		}
	}
	else
		sap->sin_addr.s_addr = htonl( INADDR_ANY );

	port = (short)strtol( sname, &endptr, 0 );
	if( *endptr == '\0' )
		sap->sin_port = htons( port );
	else
	{
		sp = getservbyname( sname, protocol );
		if( sp == NULL )
			error( 1, 0, "unknown service: %s\n", sname );
		sap->sin_port = sp->s_port;
	}
}


void init( char **argv )
{
	WSADATA		wsadata;

	( program_name = strrchr( argv[0], '\\' )) ? program_name++ : (program_name = argv[0] );
	WSAStartup( MAKEWORD( 2,2 ), &wsadata );
}


int send_string( SOCKET s, char *outstring )
{
	int		rc;

	rc = send( s, outstring, strlen( outstring ), 0 );
	total_bytes += strlen( outstring );
	return( rc ); 
}


int send_data( SOCKET s, unsigned char *data, int num_of_bytes )
{
	int		rc;

	rc = send( s, data, num_of_bytes, 0 );
	total_bytes += num_of_bytes;
	return( rc ); 
}



int main ( int argc, char **argv )
{
	FILE			*input_file;
	struct			sockaddr_in	peer;
	char			*hname;
	SOCKET			s;
	const			char	on = 1;
	char			port[256];
	unsigned short	length,i;
	unsigned char	temp[PACKET_SIZE];
	unsigned char	ascii_length[4];



	printf( "\n\n\n\nWeb page / Firmware Uploader for EMG HTTP server" );
	printf( "\n   Written by Eric Gregori\n\n" );

	if( argc != 4 )
	{
		printf( "\n\nUsage: emg_https_uploader <ip address> <filename> <key>" );
		printf( "\n\nWhere: ip address = ip address of http server" );
		printf( "\n        filename = filename of data to upload ( .egw or .egf )" );
		printf( "\n        key      = key string to unlock flash on target" );
		return(1);
	}

	if((input_file = fopen(argv[ARGC_INPUT_FILENAME], "rb")) == 0) 
	{ 
		printf("\n\nCannot open input file: %s\n", argv[ARGC_INPUT_FILENAME]); 
		return(1);
	}

	INIT();

	strcpy( port, "80" );

	hname = argv[ARGC_IP_ADDRESS];

	set_address( hname, port, &peer, "tcp" );
	s = socket( AF_INET, SOCK_STREAM, 0 );
	if( !isvalidsock( s ) )
		error( 1, errno, "socket call failed" );

	if( connect( s, (struct sockaddr *)&peer, sizeof( peer ) ) )
	{
		printf( "\n\nFailed trying to connect to ip address: %s", argv[ARGC_IP_ADDRESS] );
		error( 1, errno, "\nconnect failed" );
	}

	total_bytes = 0;

	length = fread(temp, 1, 8, input_file);

	ascii_length[0] = temp[4];
	ascii_length[1] = temp[5];
	ascii_length[2] = temp[6];
	ascii_length[3] = temp[7];

	temp[0] = 'E';
	temp[1] = 'M';
	temp[2] = 'G';
	temp[3] = ' ';
	temp[4] = '/';

	for( i=0; argv[ARGC_KEY][i]; i++ )
		temp[i+5] = argv[ARGC_KEY][i];

	temp[i+5] = ' ';

	temp[i+6] = ascii_length[0];
	temp[i+7] = ascii_length[1];
	temp[i+8] = ascii_length[2];
	temp[i+9] = ascii_length[3];


	printf( "\nSending Erase Command" );
	printf( "\nSending Unlock Key" );
	(void)send_data( s, temp, (i+10) );

	printf( "\n\nWaiting for Erase ACK" );
	while( 1 )
	{
		// Wait for Ack from board
		length = recv( s, temp, PACKET_SIZE, 0 );

//		printf( "\n%d", length );

		// If ACK contains the word Complete then send file
		if( length )
		{
			// Verify valid string
			for( i=0; i<length; i++ )
			{
				if( !temp[i] )
					break;
			}

			if( !temp[i] )
			{
				printf( "\n%s", temp );
				if( !strcmp( temp, "Erase Complete" ) )
					break;
			}
		}
	}

	rewind( input_file );

	printf( "\nSending File %s to %s\n", argv[ARGC_INPUT_FILENAME], argv[ARGC_IP_ADDRESS] );
	
	for( ; !feof(input_file); )
	{
		if ((length = fread(temp, 1, PACKET_SIZE, input_file)) == 0)
			break;			

		if ( ferror(input_file) )
			break;

		(void)send_data( s, temp, length );

		printf( "\r%d bytes sent", total_bytes ); 

	}

	temp[0] = 0;
	temp[1] = 0;
	(void)send_data( s, temp, 2 );

	printf( "\n\nWaiting for Upload Complete" );
	while( 1 )
	{
		// Wait for Ack from board
		length = recv( s, temp, PACKET_SIZE, 0 );

//		printf( "\n%d", length );

		// If ACK contains the word Complete then send file
		if( length )
		{
			// Verify valid string
			for( i=0; i<length; i++ )
			{
				if( !temp[i] )
					break;
			}

			if( !temp[i] )
			{
				printf( "\n%s", temp );
				if( !strcmp( temp, "Upload Complete" ) )
					break;
			}
		}
	}

	CLOSE(s);

	printf( "\n\n%d bytes successfully sent", total_bytes );
	printf( "\nTransmission Complete\n\n\n" );

	EXIT(0);
}
