//*****************************************************************************
// Copyright (c) 2006, Freescale Semiconductor
// Freescale Confidential Proprietary
// For use on Freescale products only.
//
// Project name: 	emg_HTTP web server for Coldfire
// Author:			Eric Gregori   (847) 651 - 1971
//		   			eric.gregori@freescale.com
//
//
//*****************************************************************************
#include <stdio.h>
#include <stdlib.h>
#include <string.h>



#define REVISION					"1.2"
#define FILENAME_LENGTH				32
#define MAX_NUMBER_OF_FILES			64
#define FFS_REVISION_NUMBER			0x0001
#define ARGC_INPUT_FILENAME			1
#define ARGC_OUTPUT_FILENAME		2
#define TRUE						1
#define FALSE						0
#define SCRATCH_SIZE				255

#define HTTP_OK_RESPONSE			"HTTP/1.1 200 OK\r\n"
#define HTTP_FAIL_RESPONSE			"HTTP/1.1 404 Not found\r\n"
#define HTTP_SERVER_NAME			"Server: EMG/1.1.0\r\n"
#define HTTP_CACHE_CONTROL			"Cache-control: no-cache\r\n"
#define HTTP_TITLE					"HTTP Server By Eric Gregori Version 1.1.0\r\n"
#define HTTP_CONTENT_TYPE			"Content-type: "
#define HTTP_CONTENT_LENGTH			"Content-length: "

#define HTTP_HTM					"text/html"
#define HTTP_TXT					"text/plain"
#define HTTP_GIF					"image/gif"
#define HTTP_XBM					"image/x-xbitmap"
#define HTTP_WAV					"audio/wav"
#define HTTP_JPEG					"image/jpeg"
#define HTTP_JAR					"application/java-archive"
#define HTTP_JS						"application/x-javascript"
#define HTTP_SWF					"application/x-shockwave-flash"
#define HTTP_PNG					"image/png"


typedef struct {
					unsigned short ffs_revision;
					unsigned short fat_pointer;
					unsigned short numoffiles;
				} HEADER;

typedef struct {
					unsigned char  filename[FILENAME_LENGTH];
					unsigned char  real_filename[FILENAME_LENGTH];
					unsigned short file_pointer;
					unsigned short file_length;
					unsigned long  file_type;
				} FILE_DESCRIPTOR; 


FILE_DESCRIPTOR	file_desc_array[MAX_NUMBER_OF_FILES];
HEADER			output_file_header;



//  Forward Declarations and Prototypes 
unsigned short copy_file_to_output( FILE *input, FILE *output );
void print_data(FILE *output, int bytenum, int data );
int write_header( FILE *output, unsigned char *filename, unsigned long *filetype );


unsigned short global_counter;


void main( int argc, char *argv[])
{
	FILE			*input_file, *output_file, *data_file; 
	unsigned short	i;
	unsigned short	number_of_files;
	unsigned short	length, header_size;
	unsigned char	tempstorage[SCRATCH_SIZE];	/* should probably malloc ... */
	unsigned char	tempstorage2[SCRATCH_SIZE];


	printf("\nStatic Flash File System Generator (Rev:%s %s)",__DATE__, __TIME__);
	printf("\n\tWritten by Eric Gregori\n");

	if( argc != 3 )
	{
		// Print usage banner
		printf("\nCorrect format:  <input filename> <output filename>\n"); 
		printf("\n");
		printf("Where input filename = file containing list of filenames\n");
		printf("\n");
		printf("Where output filename = generated filename\n");
		return;
	}

	if((input_file = fopen(argv[ARGC_INPUT_FILENAME], "r+b")) == 0) 
	{ 
		printf("Cannot open input file: %s\n", argv[ARGC_INPUT_FILENAME]); 
		return;
	}

	if((output_file = fopen(argv[ARGC_OUTPUT_FILENAME], "w")) == 0) 
	{ 
		printf("Cannot open output file: %s\n", argv[ARGC_OUTPUT_FILENAME]); 
		return;
	}
	
	fprintf(output_file,"\n");
	fprintf(output_file,"\n//***********************************************************************");
	fprintf(output_file,"\n//* Static Flash File System Generator                                  *");
	fprintf(output_file,"\n//* Written by Eric Gregori - Chicago FAE                               *");
	fprintf(output_file,"\n//*                                                                     *");
	fprintf(output_file,"\n//***********************************************************************");

	// init header
	output_file_header.ffs_revision = FFS_REVISION_NUMBER;
	output_file_header.fat_pointer  = 0;
	output_file_header.numoffiles   = 0;

	number_of_files = 0;
	length          = 0;
	do
	{
		if( fgets( tempstorage, SCRATCH_SIZE, input_file ) == NULL )
			break;

		if( (tempstorage[0] != ' ') && (tempstorage[0] != '*') )
		{
			// populate file_desc_array
			if( (strlen( tempstorage ) < FILENAME_LENGTH) &&
				(strlen( tempstorage ) > 3) )
			{
				for( i=0; tempstorage[i]; i++ )
				{
					if( (tempstorage[i] == 0x0D) || (tempstorage[i] == 0x0A) )
						tempstorage[i] = 0;
				}

				strcpy( tempstorage2, tempstorage );
				for( i=0; tempstorage2[i]; i++ )
				{
					if( tempstorage2[i] == '.' )
						tempstorage2[i] = '_';
				}

				fprintf(output_file,"\n\n\n    const unsigned char %s[] = {", tempstorage2 );

				header_size = write_header( output_file, tempstorage, &file_desc_array[number_of_files].file_type  );

				if((data_file = fopen(tempstorage, "rb")) != NULL) 
				{ 
					printf( "\nreading file: %s", tempstorage );
					file_desc_array[number_of_files].file_pointer = length;
					strcpy( file_desc_array[number_of_files].filename, tempstorage2 );
					strcpy( file_desc_array[number_of_files].real_filename, tempstorage );
					file_desc_array[number_of_files].file_length = copy_file_to_output( data_file, output_file );
					length += file_desc_array[number_of_files].file_length;
					file_desc_array[number_of_files].file_length += header_size;
					number_of_files++;
					fclose( data_file );
					fprintf(output_file, "0x00 };" );
				}
				else
				{
					printf("\nCannot open file: %s\n", tempstorage); 
				}
			}
		}
	}while( !feof(input_file) );

	if( !number_of_files )
	{
		printf( "\n\nNo filenames found in input file" );
		return;
	}

	output_file_header.fat_pointer  = length;
	output_file_header.numoffiles   = number_of_files;

#if 0
	data = (unsigned char *)file_desc_array;
	length = 0;
	for (i=0; i < (number_of_files*sizeof(FILE_DESCRIPTOR)) ;i++)
	{
		print_data( output_file, length, data[i] );
		length++; 
	} 
#endif

	fprintf( output_file, "\n\n    const char *emg_static_ffs_filenames[] =   {" );
	for( i=0; i<(number_of_files-1); i++ )
		fprintf( output_file, "\n                                               \"%s\",", file_desc_array[i].real_filename );
	fprintf( output_file, "\n                                               \"%s\" ", file_desc_array[i].real_filename );
	fprintf( output_file, "\n                                             };" );

	fprintf( output_file, "\n\n    const unsigned char *emg_static_ffs_ptrs[] =   {" );
	for( i=0; i<(number_of_files-1); i++ )
		fprintf( output_file, "\n                                               %s,", file_desc_array[i].filename );
	fprintf( output_file, "\n                                               %s", file_desc_array[i].filename );
	fprintf( output_file, "\n                                             };" );

	fprintf( output_file, "\n\n    const unsigned short emg_static_ffs_len[] =     {" );
	for( i=0; i<(number_of_files-1); i++ )
		fprintf( output_file, "\n                                               %d,", file_desc_array[i].file_length );
	fprintf( output_file, "\n                                               %d", file_desc_array[i].file_length );
	fprintf( output_file, "\n                                             };" );

	fprintf( output_file, "\n\n    const unsigned long emg_static_ffs_type[] =     {" );
	for( i=0; i<(number_of_files-1); i++ )
		fprintf( output_file, "\n                               0x%x,", file_desc_array[i].file_type );
	fprintf( output_file, "\n                                   0x%x", file_desc_array[i].file_type );
	fprintf( output_file, "\n                                             };" );

//	fprintf(output_file, "\n\nunsigned long ffs_revision = 0x%X", output_file_header.ffs_revision  );
//	fprintf(output_file, "\n\nunsigned long fat_pointer = 0x%X", output_file_header.fat_pointer  );
//	fprintf(output_file, "\n\nunsigned long numoffiles = 0x%X", output_file_header.numoffiles  );

	fprintf( output_file, "\n\n    const unsigned char emg_static_ffs_nof = %d;\n\n", number_of_files ); 
	_fcloseall( );
}


void print_data(FILE *output, int bytenum, int data )
{
	int divisor;


	global_counter++;
	divisor = 10;
	if( bytenum == 0 )
		fprintf(output, "\n    0x%2.2X,", data);
	else
	{
		if( (bytenum % divisor) == 0 )
			fprintf(output, "\n    0x%2.2X,", data);
		else
			fprintf(output,"0x%2.2X,", data);
	}
}


void fprintf_output( FILE *output, unsigned char *string )
{
	int		chars;

	for( chars = 0; string[chars]; chars++ )
		print_data( output, chars, string[chars] );

}


unsigned short copy_file_to_output( FILE *input, FILE *output )
{
	unsigned short	i;
	unsigned short	length, total_count;
	unsigned char	temp[255];


	for( total_count=0, total_count=0; !feof(input); )
	{
		if ((length = fread(temp, 1, 255, input)) == 0)
			break;			

		if ( ferror(input) )
			break;

		for (i=0; i < length ;i++)
		{
			print_data( output, total_count, temp[i] );
			total_count++; 
		} 
	}

	return( total_count );
}


int write_header( FILE *output, unsigned char *filename, unsigned long *filetype )
{
	FILE			*P_file;
	unsigned int	length, file_size;
	unsigned char	nstring[255];


	global_counter = 0;
	fprintf_output( output, HTTP_OK_RESPONSE );
	fprintf_output( output, HTTP_SERVER_NAME );
	fprintf_output( output, HTTP_CACHE_CONTROL );
	fprintf_output( output, HTTP_TITLE );
	fprintf_output( output, HTTP_CONTENT_TYPE );

	for( length=0; length<64; length++ )
	{
		if( filename[length] == '.' )
			break;
	}
					
	if( strncmp( &filename[length+1], "txt", 3 ) == 0 )
	{
		fprintf_output( output, HTTP_TXT );
		*filetype = (unsigned long)'text';
	}
	else if( strncmp( &filename[length+1], "gif", 3 ) == 0 )
	{
		fprintf_output( output, HTTP_GIF );
		*filetype = (unsigned long)'giff';
	}
	else if( strncmp( &filename[length+1], "wav", 3 ) == 0 )
	{
		fprintf_output( output, HTTP_WAV );
		*filetype = (unsigned long)'wave';
	}
	else if( strncmp( &filename[length+1], "jpg", 3 ) == 0 )
	{
		fprintf_output( output, HTTP_JPEG );
		*filetype = (unsigned long)'jpeg';
	}
	else if( strncmp( &filename[length+1], "jar", 3 ) == 0 )
	{
		fprintf_output( output, HTTP_JAR );
		*filetype = (unsigned long)'jarr';
	}
	else if( strncmp( &filename[length+1], "js", 2 ) == 0 )
	{
		fprintf_output( output, HTTP_JS );
		*filetype = (unsigned long)'jsjs';
	}
	else if( strncmp( &filename[length+1], "swf", 3 ) == 0 )
	{
		fprintf_output( output, HTTP_SWF );
		*filetype = (unsigned long)'swff';
	}
	else if( strncmp( &filename[length+1], "png", 3 ) == 0 )
	{
		fprintf_output( output, HTTP_PNG );
		*filetype = (unsigned long)'pngg';
	}
	else
	{
		fprintf_output( output, HTTP_HTM );
		*filetype = (unsigned long)'html';
	}

	fprintf_output( output, "\r\n" );
	fprintf_output( output, HTTP_CONTENT_LENGTH );
			
	P_file = fopen( filename, "rb" );
	if( P_file )
	{
  		fseek (P_file,0, SEEK_END);
  		file_size = ftell (P_file);

		/* complete header */
		sprintf( nstring, "%d\r\n\r\n", file_size );
		fprintf_output( output, nstring );
		rewind( P_file );
		fclose( P_file );
		return(global_counter);
	}
		
	return(0);
}


